@@ -1,8 +1,7 @@
 /*
  *      dialogs.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005 The Geany contributors
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -33,6 +32,7 @@
 #include "build.h"
 #include "document.h"
 #include "encodings.h"
+#include "encodingsprivate.h"
 #include "filetypes.h"
 #include "main.h"
 #include "support.h"
@@ -491,6 +491,7 @@ static gboolean handle_save_as(const gchar *utf8_filename, gboolean rename_file)
 	GeanyDocument *doc = document_get_current();
 	gboolean success = FALSE;
 
+	g_return_val_if_fail(doc != NULL, FALSE);
 	g_return_val_if_fail(!EMPTY(utf8_filename), FALSE);
 
 	if (doc->file_name != NULL)
@@ -1057,11 +1058,11 @@ static void on_dialog_input(const gchar *str, gpointer data)
 
 /** Asks the user for text input.
  * @param title Dialog title.
- * @param parent The currently focused window, usually @c geany->main_widgets->window.
+ * @param parent @nullable The currently focused window, usually @c geany->main_widgets->window.
  * 	@c NULL can be used but is discouraged due to window manager effects.
- * @param label_text Label text, or @c NULL.
- * @param default_text Text to display in the input field, or @c NULL.
- * @return New copy of user input or @c NULL if cancelled.
+ * @param label_text @nullable Label text, or @c NULL.
+ * @param default_text @nullable Text to display in the input field, or @c NULL.
+ * @return @nullable New copy of user input or @c NULL if cancelled.
  * @since 0.20. */
 GEANY_API_SYMBOL
 gchar *dialogs_show_input(const gchar *title, GtkWindow *parent, const gchar *label_text,
@@ -1290,7 +1291,7 @@ void dialogs_show_file_properties(GeanyDocument *doc)
 /* extra_text can be NULL; otherwise it is displayed below main_text.
  * if parent is NULL, main_widgets.window will be used
  * btn_1, btn_2, btn_3 can be NULL.
- * returns response_1, response_2 or response_3 */
+ * returns response_1, response_2, response_3, or GTK_RESPONSE_DELETE_EVENT if the dialog was discarded */
 static gint show_prompt(GtkWidget *parent,
 		const gchar *btn_1, GtkResponseType response_1,
 		const gchar *btn_2, GtkResponseType response_2,
@@ -1344,9 +1345,7 @@ static gint show_prompt(GtkWidget *parent,
 	if (btn_1 != NULL)
 		gtk_dialog_add_button(GTK_DIALOG(dialog), btn_1, response_1);
 
-	/* For a cancel button, use cancel response so user can press escape to cancel */
-	btn = gtk_dialog_add_button(GTK_DIALOG(dialog), btn_2,
-		utils_str_equal(btn_2, GTK_STOCK_CANCEL) ? GTK_RESPONSE_CANCEL : response_2);
+	btn = gtk_dialog_add_button(GTK_DIALOG(dialog), btn_2, response_2);
 	/* we don't want a default, but we need to override the apply button as default */
 	gtk_widget_grab_default(btn);
 	gtk_dialog_add_button(GTK_DIALOG(dialog), btn_3, response_3);
@@ -1354,8 +1353,6 @@ static gint show_prompt(GtkWidget *parent,
 	ret = gtk_dialog_run(GTK_DIALOG(dialog));
 	gtk_widget_destroy(dialog);
 
-	if (ret == GTK_RESPONSE_CANCEL)
-		ret = response_2;
 	return ret;
 }
 
